==> subtitlecomposer-0.4.1.tar.bz2: subtitlecomposer-0.4.1/src/main/utils/find.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "find.h"

#include <kdebug.h>
#include <klocale.h>

#include <kmessagebox.h>

#include <kfinddialog.h>
#include <kfind.h>


using namespace SubtitleComposer;

Find::Find( QWidget* parent ):
	QObject( parent ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_feedPrimaryNext( true ),
	m_find( 0 ),
	m_iterator( 0 ),
	m_dataLine( 0 )
{
	m_dialog = new KFindDialog( parent, "FindDialog" );
	m_dialog->setHasSelection( true );
	m_dialog->setHasCursor( true );
	m_dialog->setOptions( m_dialog->options() | KFindDialog::FromCursor );
}

Find::~Find()
{
	delete m_dialog;

	setSubtitle( 0 );
}

void Find::invalidate()
{
	delete m_find;
	m_find = 0;

	delete m_iterator;
	m_iterator = 0;

	m_feedPrimaryNext = true;

	// NOTE a line shoudn't be deleted before being removed from the subtitle
	// so there's no risk of being left with an invalid referece as this
	// method is called when a line is removed from the subtitle and before
	// anyone has a chance to delete the line.
	if ( m_dataLine )
	{
		disconnect( m_dataLine, SIGNAL( primaryTextChanged(const QString&) ),
					this, SLOT( onLinePrimaryTextChanged(const QString&) ) );
		disconnect( m_dataLine, SIGNAL( secondaryTextChanged(const QString&) ),
					this, SLOT( onLineSecondaryTextChanged(const QString&) ) );
		m_dataLine = 0;
	}
}

QWidget* Find::parentWidget()
{
	return static_cast<QWidget*>( parent() );
}

void Find::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}

	m_subtitle = subtitle;

	invalidate();

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}
}

void Find::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	invalidate();
}

void Find::find( const RangeList& selectionRanges, unsigned currentIndex, bool findBackwards )
{
	invalidate();

	if ( ! m_subtitle || ! m_subtitle->linesCount() )
		return;

	m_dialog->setOptions(
		findBackwards ?
			m_dialog->options() | KFindDialog::FindBackwards :
			m_dialog->options() & ~KFindDialog::FindBackwards
	);

	if ( m_dialog->exec() != QDialog::Accepted )
		return;

	m_find = new KFind( m_dialog->pattern(), m_dialog->options(), 0 );
	m_find->closeFindNextDialog();

	connect( m_find, SIGNAL( highlight(const QString&,int,int) ),
			 this, SLOT( onHighlight(const QString&,int,int) ) );

	if ( m_dialog->options() & KFindDialog::SelectedText )
	{
		m_iterator = new SubtitleIterator( *m_subtitle, selectionRanges );
		if ( m_iterator->index() < 0 ) // Invalid index means no lines in selectionRanges
			return;
	}
	else
		m_iterator = new SubtitleIterator( *m_subtitle );

	if ( m_dialog->options() & KFindDialog::FromCursor )
		m_iterator->toIndex( currentIndex );

	m_firstIndex = m_iterator->index();

	m_find->setPattern( m_dialog->pattern() );
	m_find->setOptions( m_dialog->options() );

	m_found = false;

	advance();
}

bool Find::findNext()
{
	if ( ! m_iterator )
		return false;

	m_find->setOptions( m_find->options() & ~KFindDialog::FindBackwards );

	advance();
	return true;
}

bool Find::findPrevious()
{
	if ( ! m_iterator )
		return false;

	m_find->setOptions( m_find->options() | KFindDialog::FindBackwards );

	advance();
	return true;
}

void Find::advance()
{
	KFind::Result res = KFind::NoMatch;

	bool selection = m_find->options() & KFindDialog::SelectedText;
	bool backwards = m_find->options() & KFindDialog::FindBackwards;

	do
	{
		if ( m_find->needData() )
		{
			if ( m_dataLine )
			{
				disconnect( m_dataLine, SIGNAL( primaryTextChanged(const QString&) ),
							this, SLOT( onLinePrimaryTextChanged(const QString&) ) );
				disconnect( m_dataLine, SIGNAL( secondaryTextChanged(const QString&) ),
							this, SLOT( onLineSecondaryTextChanged(const QString&) ) );
			}
			m_dataLine = m_iterator->current();

			if ( m_translationMode )
			{
				m_find->setData( m_feedPrimaryNext ? m_dataLine->primaryText() : m_dataLine->secondaryText() );
				m_feedPrimaryNext = ! m_feedPrimaryNext;
			}
			else
				m_find->setData( m_dataLine->primaryText() );

			connect( m_dataLine, SIGNAL( primaryTextChanged(const QString&) ),
					 this, SLOT( onLinePrimaryTextChanged(const QString&) ) );
			connect( m_dataLine, SIGNAL( secondaryTextChanged(const QString&) ),
					 this, SLOT( onLineSecondaryTextChanged(const QString&) ) );
		}

		res = m_find->find();

		if ( res == KFind::Match )
			m_found = true;
		else if ( ! m_translationMode || m_feedPrimaryNext )
		{
			if ( backwards )
				--(*m_iterator);
			else
				++(*m_iterator);

			if ( ! m_found ) // special case
			{
				if ( m_firstIndex == m_iterator->index() ||
					(backwards ?
						(m_firstIndex == m_iterator->lastIndex() && m_iterator->index() == SubtitleIterator::BEHIND_FIRST) :
						(m_firstIndex == m_iterator->firstIndex() && m_iterator->index() == SubtitleIterator::AFTER_LAST)) )
				{
					KMessageBox::sorry(
						parentWidget(),
						i18n( "No instances of '%1' found!" ).arg( m_find->pattern() ),
						i18n( "Find" )
					);

					break;
				}
			}

			if ( m_iterator->index() < 0 )
			{
				if ( backwards )
					m_iterator->toLast();
				else
					m_iterator->toFirst();

				if ( KMessageBox::warningContinueCancel(
						parentWidget(),
						backwards ?
							(selection ?
								i18n( "Beginning of selection reached.\nContinue from the end?" ) :
								i18n( "Beginning of subtitle reached.\nContinue from the end?" )) :
							(selection ?
								i18n( "End of selection reached.\nContinue from the beginning?" ) :
								i18n( "End of subtitle reached.\nContinue from the beginning?" )),
						i18n( "Find" )
						) != KMessageBox::Continue )
					break;
			}
		}
	}
	while ( res != KFind::Match );
}

void Find::onHighlight( const QString&, int matchingIndex, int matchedLength )
{
	bool primary = ! m_translationMode || ! m_feedPrimaryNext;

	emit found( m_iterator->current(), primary, matchingIndex, matchingIndex + matchedLength - 1 );
}

void Find::onLinePrimaryTextChanged( const QString& text )
{
	if ( ! m_translationMode || ! m_feedPrimaryNext )
		m_find->setData( text );
}

void Find::onLineSecondaryTextChanged( const QString& text )
{
	if ( m_translationMode && m_feedPrimaryNext )
		m_find->setData( text );
}

#include "find.moc"

==> subtitlecomposer-0.4.2.tar.bz2: subtitlecomposer-0.4.2/src/main/utils/find.cpp <==
/***************************************************************************
 *   Copyright (C) 2007-2008 by Sergio Pistone                             *
 *   sergio_pistone@yahoo.com.ar                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "find.h"
#include <subtitleiterator.h>

#include <kdebug.h>
#include <klocale.h>

#include <kmessagebox.h>

#include <kfinddialog.h>
#include <kfind.h>


using namespace SubtitleComposer;

Find::Find( QWidget* parent ):
	QObject( parent ),
	m_subtitle( 0 ),
	m_translationMode( false ),
	m_feedPrimaryNext( true ),
	m_find( 0 ),
	m_iterator( 0 ),
	m_dataLine( 0 )
{
	m_dialog = new KFindDialog( parent, "FindDialog" );
	m_dialog->setHasSelection( true );
	m_dialog->setHasCursor( true );
	m_dialog->setOptions( m_dialog->options() | KFindDialog::FromCursor );
}

Find::~Find()
{
	delete m_dialog;

	setSubtitle( 0 );
}

void Find::invalidate()
{
	delete m_find;
	m_find = 0;

	delete m_iterator;
	m_iterator = 0;

	m_feedPrimaryNext = true;

	// NOTE a line shoudn't be deleted before being removed from the subtitle
	// so there's no risk of being left with an invalid referece as this
	// method is called when a line is removed from the subtitle and before
	// anyone has a chance to delete the line.
	if ( m_dataLine )
	{
		disconnect( m_dataLine, SIGNAL( primaryTextChanged(const SString&) ),
					this, SLOT( onLinePrimaryTextChanged(const SString&) ) );
		disconnect( m_dataLine, SIGNAL( secondaryTextChanged(const SString&) ),
					this, SLOT( onLineSecondaryTextChanged(const SString&) ) );
		m_dataLine = 0;
	}
}

QWidget* Find::parentWidget()
{
	return static_cast<QWidget*>( parent() );
}

void Find::setSubtitle( Subtitle* subtitle )
{
	if ( m_subtitle )
	{
		disconnect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		disconnect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}

	m_subtitle = subtitle;

	invalidate();

	if ( m_subtitle )
	{
		connect( m_subtitle, SIGNAL( linesInserted(unsigned,unsigned) ), this, SLOT( invalidate() ) );
		connect( m_subtitle, SIGNAL( linesRemoved(unsigned,unsigned) ), this, SLOT( invalidate() ) );
	}
}

void Find::setTranslationMode( bool enabled )
{
	m_translationMode = enabled;

	invalidate();
}

void Find::find( const RangeList& selectionRanges, unsigned currentIndex, bool findBackwards )
{
	invalidate();

	if ( ! m_subtitle || ! m_subtitle->linesCount() )
		return;

	m_dialog->setOptions(
		findBackwards ?
			m_dialog->options() | KFindDialog::FindBackwards :
			m_dialog->options() & ~KFindDialog::FindBackwards
	);

	if ( m_dialog->exec() != QDialog::Accepted )
		return;

	m_find = new KFind( m_dialog->pattern(), m_dialog->options(), 0 );
	m_find->closeFindNextDialog();

	connect( m_find, SIGNAL( highlight(const QString&,int,int) ),
			 this, SLOT( onHighlight(const QString&,int,int) ) );

	if ( m_dialog->options() & KFindDialog::SelectedText )
	{
		m_iterator = new SubtitleIterator( *m_subtitle, selectionRanges );
		if ( m_iterator->index() < 0 ) // Invalid index means no lines in selectionRanges
			return;
	}
	else
		m_iterator = new SubtitleIterator( *m_subtitle );

	if ( m_dialog->options() & KFindDialog::FromCursor )
		m_iterator->toIndex( currentIndex );

	m_firstIndex = m_iterator->index();

	m_find->setPattern( m_dialog->pattern() );
	m_find->setOptions( m_dialog->options() );

	m_found = false;

	advance();
}

bool Find::findNext()
{
	if ( ! m_iterator )
		return false;

	m_find->setOptions( m_find->options() & ~KFindDialog::FindBackwards );

	advance();
	return true;
}

bool Find::findPrevious()
{
	if ( ! m_iterator )
		return false;

	m_find->setOptions( m_find->options() | KFindDialog::FindBackwards );

	advance();
	return true;
}

void Find::advance()
{
	KFind::Result res = KFind::NoMatch;

	bool selection = m_find->options() & KFindDialog::SelectedText;
	bool backwards = m_find->options() & KFindDialog::FindBackwards;

	do
	{
		if ( m_find->needData() )
		{
			if ( m_dataLine )
			{
				disconnect( m_dataLine, SIGNAL( primaryTextChanged(const SString&) ),
							this, SLOT( onLinePrimaryTextChanged(const SString&) ) );
				disconnect( m_dataLine, SIGNAL( secondaryTextChanged(const SString&) ),
							this, SLOT( onLineSecondaryTextChanged(const SString&) ) );
			}
			m_dataLine = m_iterator->current();

			if ( m_translationMode )
			{
				const SString& text = m_feedPrimaryNext ? m_dataLine->primaryText() : m_dataLine->secondaryText();
				m_find->setData( text.string() );
				m_feedPrimaryNext = ! m_feedPrimaryNext;
			}
			else
				m_find->setData( m_dataLine->primaryText().string() );

			connect( m_dataLine, SIGNAL( primaryTextChanged(const SString&) ),
					 this, SLOT( onLinePrimaryTextChanged(const SString&) ) );
			connect( m_dataLine, SIGNAL( secondaryTextChanged(const SString&) ),
					 this, SLOT( onLineSecondaryTextChanged(const SString&) ) );
		}

		res = m_find->find();

		if ( res == KFind::NoMatch && (! m_translationMode || m_feedPrimaryNext) )
		{
			if ( backwards )
				--(*m_iterator);
			else
				++(*m_iterator);

			if ( ! m_found ) // special case
			{
				if ( m_firstIndex == m_iterator->index() ||
					(backwards ?
						(m_firstIndex == m_iterator->lastIndex() && m_iterator->index() == SubtitleIterator::BEHIND_FIRST) :
						(m_firstIndex == m_iterator->firstIndex() && m_iterator->index() == SubtitleIterator::AFTER_LAST)) )
				{
					KMessageBox::sorry(
						parentWidget(),
						i18n( "No instances of '%1' found!" ).arg( m_find->pattern() ),
						i18n( "Find" )
					);

					break;
				}
			}

			if ( m_iterator->index() < 0 )
			{
				if ( backwards )
					m_iterator->toLast();
				else
					m_iterator->toFirst();

				if ( KMessageBox::warningContinueCancel(
						parentWidget(),
						backwards ?
							(selection ?
								i18n( "Beginning of selection reached.\nContinue from the end?" ) :
								i18n( "Beginning of subtitle reached.\nContinue from the end?" )) :
							(selection ?
								i18n( "End of selection reached.\nContinue from the beginning?" ) :
								i18n( "End of subtitle reached.\nContinue from the beginning?" )),
						i18n( "Find" )
						) != KMessageBox::Continue )
					break;
			}
		}
	}
	while ( res == KFind::NoMatch );
}

void Find::onHighlight( const QString&, int matchingIndex, int matchedLength )
{
	m_found = true;

	bool primary = ! m_translationMode || ! m_feedPrimaryNext;

	emit found( m_iterator->current(), primary, matchingIndex, matchingIndex + matchedLength - 1 );
}

void Find::onLinePrimaryTextChanged( const SString& text )
{
	if ( ! m_translationMode || ! m_feedPrimaryNext )
		m_find->setData( text.string() );
}

void Find::onLineSecondaryTextChanged( const SString& text )
{
	if ( m_translationMode && m_feedPrimaryNext )
		m_find->setData( text.string() );
}

#include "find.moc"
